//******************************************************************************
//   RTC and Display PWM routines
// 
//   written for MSP430F1101A
//
//   This program is written as a courtesy to customers to aid development of  
//   their products. Please see the legal notice at the end of this file. 
//   
//   RTC handler
//     - counts 50/60Hz or seconds
//     - notify host of SEC or MIN change
//
//   SPI handler
//     - this is a slave SPI
//     - CS inits counters
//     - clock is from master
//     - data is bi-directional
//
//
//                        MSP430F1101A
//                   --------------------
//            ------|XIN                 |  
// 32KHz xtal       |                    |
//            ------|XOUT                |  
//                  |                    |
//        60Hz in --|P1.3/TA2            |
//                  |                    |
//                  |                P2.4|--- seconds
//                  |                P2.5|--- minutes
//                  |                    |
//                  |                P1.1|--- power good
//        SPI_RES --| P2.1               |
//        SPI_CLK --| P2.2               |
//       SPI_DATA --| P2.3               |
//                  |                    |
//
//   Mike Kaelin
//   Texas Instruments, Inc
//   April 2003
//******************************************************************************

#include <msp430x11x1.h>

//------------------------------------------------------------------------------
// variable definitions
//------------------------------------------------------------------------------ 

// real time clock variables in RAM
char HZ_CNT, SEC, MIN, HR, FORMAT;

// SPI variables in RAM
char BIT_CNT, BYTE_CNT, SPI_DIR;

// real time clock input frequency
// 0 = 1 second input, 50 & 60 are 50/60 Hz
char CK_Input;

// pin definitions  
#define SPI_RES           (0x02)
#define SPI_CLK           (0x04)
#define SPI_DATA          (0x08)

// command definitions (using 0/1/2/4/8/ffff/fffe will invoke the constant 
//                        generator, which helps keep code size down)
// (commands are R or W relative to master)
#define SPI_WRITE_1               (0x01)
#define SPI_WRITE_2               (0x02)
#define SPI_READ_1                (0x04)
#define SPI_READ_2                (0x08)

// data direction, from slave point of view
#define SPI_IN                    (0x01)
#define SPI_OUT                   (0x02)

// longest number of bytes in a message
#define MAXLENGTH 4

// variables
unsigned char buffer[MAXLENGTH];


// byte locations
#define COMMAND                   (0x00)
#define BYTE1                     (0x01)
#define BYTE2                     (0x02)
#define BYTE3                     (0x03)
// message status



//------------------------------------------------------------------------------
// main()
//
// - initializes ports and timers
// - puts micro to sleep
// - when micro wakes up for RTC seconds, handles clock counters
// - puts micro back to sleep after RTC handler
//------------------------------------------------------------------------------

void main(void)
{ 
// Initialization: this is the first power up

// Stop WDT
WDTCTL = WDTPW + WDTHOLD;             

// init port I/O
// (***** should be changed by customer for actual requirements)
P1DIR = 0xff;                                   // all outputs for now 
P2DIR = 0xff & ~SPI_RES & ~SPI_CLK & ~SPI_DATA;  // all outputs except SPI lines

P2IE |= SPI_RES + SPI_CLK;                       // enable interrupts on CS and CLK
P2IES |= 0;                                     // interrupt on low-to-high edge
P2IFG = 0;                                      // clear interrupts

// init SPI counters
BIT_CNT = 0;
BYTE_CNT = 0;

// init RTC time, set up RTC timer

HR = 4;
SEC = 45;
MIN = 49;
HZ_CNT = 0;

FORMAT = 12; //12 or 24 hour format

//; init 50/60Hz flag
//; (Flag is in RAM,initialized to 0, assume off until 50/60Hz interrupt received
CK_Input = 0;                  

// Initialize timer for RTC
TACTL = TASSEL0 + TACLR;              // ACLK, clear TAR
CCTL0 = CCIE;                         // CCR0 interrupt enabled
CCR0 = 0X7FFF;
TACTL |= MC0;                         // Start Timer_a in upmode
_EINT();                              // Enable interrupts

// init comm 


// put micro to sleep, wait for 50/60Hz, comm, and timer interrupts
for (;;)                              
   {
    _BIS_SR(LPM3_bits);                 // Enter LPM3

    if (CK_Input)                       // CK_input is 0 for 1Hz clock
      {                                 // or 50/60 for 50/60 Hz input
       HZ_CNT++;
       if (HZ_CNT == CK_Input)
          {
          HZ_CNT = 0;
          SEC++;
          }
       }
    else 
      {
      SEC++;
// Seconds notify:
      P1OUT ^= 0x01;                        // Toggle P1.0

      }
   if (SEC == 60)
     {
     SEC = 0;
     MIN++;
     }
// Minute notify:
   if (MIN == 60) 
     {
     MIN = 0;
     HR++;
     if (HR == FORMAT+1) HR=0;
     }
  }  
}


//------------------------------------------------------------------------------
// Interrupt ISR's
//
// PORT1_VECTOR         0xFFE4 Port 1 */
// PORT2_VECTOR         0xFFE6 Port 2 */
// TIMERA1_VECTOR       0xFFF0 Timer A CC1-2, TA */
// TIMERA0_VECTOR       0xFFF2 Timer A CC0 */
// WDT_VECTOR           0xFFF4 Watchdog Timer */
// COMPARATORA_VECTOR   0xFFF6 Comparator A */
// NMI_VECTOR           0xFFFC Non-maskable */
// RESET_VECTOR         0xFFFE Reset [Highest Priority] */

// Port  interrupt ISR
interrupt[PORT2_VECTOR] void port2 (void)
{
if (P2IFG & SPI_RES)                          // An SPI CS rising edge inits the interface
  {
  P2IFG &= ~SPI_RES;                          // reset the RES interrupt
  BIT_CNT = 0;
  BYTE_CNT = 0;
  SPI_DIR = SPI_IN;
  }
else if (P2IFG  & SPI_CLK)                    // handle a CLK rising edge 
  {
  char byte_in;
  P2IFG &= ~SPI_CLK;                          // reset the CLK interrupt
  byte_in = P2IN;                             // get copy of input port data

  buffer[BYTE_CNT] = buffer[BYTE_CNT] << 1;   // shift, get ready for next bit
  if (SPI_DIR == SPI_IN)
    {
    if (byte_in & SPI_DATA)                   // IN: get the bit and store 
      buffer[BYTE_CNT] = buffer[BYTE_CNT] |= 1;
    else buffer[BYTE_CNT] &= 0xfe;              
    }
  else
    {
    if (buffer[BYTE_CNT] & 0x80)              // OUT: set up the data for output
      P2OUT |= SPI_DATA; else P2OUT &= ~SPI_DATA; 
    }  
    
  BIT_CNT++;
  if (BIT_CNT == 8)                           // when BIT_CNT = 8, handle byte
    {                                          
    BIT_CNT = 0;                              // reset bit counter
    if (BYTE_CNT == COMMAND)                  // handle command byte
      {                                       // SPI_WRITE: IN flag is already set
      switch(buffer[COMMAND])                 // SPI_READ: set OUT flag, prepare buffers
        {
        case SPI_READ_1:                     // SPI_READ_1 will send the time
          SPI_DIR = SPI_OUT;
          P2DIR |= SPI_DATA;                  // change DATA to output
          buffer[BYTE1] = HR; 
          buffer[BYTE2] = MIN;
          buffer[BYTE3] = SEC;
          break;
        case SPI_READ_2:                      // SPI_READ_2 will send port inputs 
          SPI_DIR = SPI_OUT;                  // ... may want to change this
          P2DIR |= SPI_DATA;                  // change DATA to output
          buffer[BYTE1] = P1IN;
          buffer[BYTE2] = P2IN;
          buffer[BYTE3] = 0;
          break;
        }             
      }
    BYTE_CNT++;
    if (BYTE_CNT == MAXLENGTH)                // message finished?
      {
      P2DIR &= ~SPI_DATA;                     // change DATA back to input
      switch(buffer[COMMAND])                  
        {                                     // SPI_WRITE: handle data
        case SPI_WRITE_1:                     // SPI_WRITE_1 will load the time
          HR = buffer[BYTE1]; 
          MIN = buffer[BYTE2];
          SEC = buffer[BYTE3];
          break;
        case SPI_WRITE_2:
          //  set port output bits here...
          _NOP();
          break;
         }
      BYTE_CNT = 0;                           // SPI_READ: just end
      BIT_CNT = 0;
      SPI_DIR = SPI_IN;
      }
    }
  }
}

// Timer A0 interrupt service routine
interrupt[TIMERA0_VECTOR] void Timer_A (void)
{
_BIC_SR_IRQ(LPM3_bits);           // Clear LPM3 bits from 0(SR)
                                  // (jump back into the program)
}






// Legal notice: 
//******************************************************************************
// THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
// REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
// FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
// COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
// TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
// POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
// INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
// YOUR USE OF THE PROGRAM.
//
// IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
// CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
// THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
// OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
// OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
// EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
// REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
// OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
// USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
// AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
// YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
// (U.S.$500).
//
