//*******************************************************************************
//  SPI communications tester - using MSP-FET430P140 Demo 
//
//  Description: SPI master to send/receive data to SPI slave
//   - master generates clock, data is bi-directional
//   - master generates an SPI reset pulse before starting
//   - master sets up data before clock rising edge
//   - master waits after rising clock edge, then reads data
//   - written in C so it can be ported 
//   - written with delay loops instead of timer to aid porting
//   - commands, data, and lengths will be in variables to make changes easy
//
//                MSP430F149
//             -----------------
//         /|\|              XIN|- 
//          | |                 |  
//          --|RST          XOUT|-
//            |                 |
//            |             P1.0|--> Reset (also LED)
//            |                 |
//            |             P1.1|---> clock
//            |             p1.2|<--> data
//            |                 |
//            |                 |
//
//  Mike Kaelin
//  Texas Instruments, Inc
//  May 2003
//******************************************************************************

#include  <msp430x14x.h>

// longest number of bytes in a message
#define MAXLENGTH 4

// delay times to allow slave to react to clock edge
// with these delays, total time is about 6mS per transmission
// repeats about every 10mS
//
// about 100uS clock pulses:
#define DELAY1 5       
// about 250 uS between bytes:
#define DELAY2 10 
// about 4 ms between 4-byte transmissions:
#define DELAY3 1000  

// variables
unsigned char buffer[MAXLENGTH];

// pin definitions  
#define RES           (0x01)
#define CLK           (0x02)
#define DATA          (0x04)

// command definitions (using 0/1/2/4/8/ffff/fffe will invoke the constant 
//                        generator, which helps keep code size down)
#define SPI_WRITE_1               (0x01)
#define SPI_WRITE_2               (0x02)
#define SPI_READ_1                (0x04)
#define SPI_READ_2                (0x08)


// byte locations
#define COMMAND                   (0x00)
#define BYTE1                     (0x01)
#define BYTE2                     (0x02)
#define BYTE3                     (0x03)

void wait (int cycles)
{
int i;
for (i=cycles;i>0;i--);                   // just count off 25 cycles
_NOP();                                   // a place to breakpoint for debugging
}


// SPI_out()
// starts with buffer[0] and sends qty bytes
//
//
//
void SPI_out (char qty)
{
int i;
P1DIR |= CLK + DATA;                      // LED, CLK, DATA are outputs
for (i=0; i<qty; i++)                                         
  {
  int j;
  for (j=8;j>0;j--)
    {
    if (buffer[i] & 0x80) 
      P1OUT |= DATA; else P1OUT &= ~DATA; // set up the data
    buffer[i] = buffer[i] << 1;           // shift bits for the next cycle
    P1OUT |= CLK;                         // clock high
    wait(DELAY1);                         // wait, give slave time to store bit
    P1OUT &= ~CLK;                        // clock low
    wait(DELAY1);                         // wait, make clock 50% duty cycle
    }    
  P1OUT &= ~DATA;                         // clear data during the delay
  wait(DELAY2);                           // add an extra delay between bytes
  }
P1DIR = 0;                                // make everything inputs again
}

// SPI_in()
// clocks "qty" bytes of data in, first byte stored in buffer[1]
//

void SPI_in (char qty)
{
int i;
P1DIR |= RES + CLK;                       // RES, CLK are outputs, DATA is an input
for (i=0; i<qty; i++)                                         
  {
  int j;
  for (j=8;j>0;j--)
    {
    char byte_in;
    P1OUT |= CLK;                         // clock high
    wait(DELAY1);                         // delay to let slave change the data
    byte_in = P1IN;                       // get copy of input port data
    buffer[i+1] = buffer[i+1] << 1;       // shift, get ready for next bit
    if (byte_in & DATA)
      buffer[i+1] = buffer[i+1] |= 1;
      else buffer[i+1] &= 0xfe;           // read the bit and store (skip command location)
    P1OUT &= ~CLK;                        // clock low
    wait(DELAY1);                         // make clock 50% duty cycle
    }    
  wait(DELAY2);                           // add an extra delay between bytes
  }
P1DIR &= ~DATA;                                // make DATA input again
}

void main(void)
{ 
WDTCTL = WDTPW + WDTHOLD;                 // Stop watchdog timer
                                          // if this line is commented out, it will
                                          // repeat every 30mS or so

P1DIR = 0;                                // all inputs, start all low
P1DIR |= RES + CLK;                       // RES, CLK are outputs, DATA is an input
P1OUT |= 0x01;                            // Toggle P1.0 (LED) just as a test flag
P1OUT = 0;


//SPI write - write one command byte, write 3 data bytes
// MAXLENGTH is defined as 4 at top of file 

P1OUT |= RES;                             // SPI reset command
wait(DELAY1);                             // delay to let slave reset the SPI
P1OUT &= ~RES;                            // end of reset pulse    

buffer[COMMAND] = SPI_WRITE_1;            // command: set clock
buffer[BYTE1] = 3;                        // Hr
buffer[BYTE2] = 24;                       // Min
buffer[BYTE3] = 35;                       // Sec
SPI_out(MAXLENGTH);

while (1)                                 // repeat clock read forever, about 30ms loop
  {
  // SPI read - send one command byte, read 3 data bytes
  buffer[COMMAND] = SPI_READ_1;             // command: read clock
  P1OUT |= RES;                             // SPI reset command
  wait(DELAY2);                             // delay to let slave reset the SPI
  P1OUT &= ~RES;                            // end of reset pulse    
  SPI_out(1);
  SPI_in(3);                                // read 3 bytes
  wait(DELAY3);
  }                              

}
